#!/usr/bin/perl 
#==================================================================================
# I-TASSER-MR uses this programe to compute AVS score and produce search models based on
# AVS score. These models can be used  to phenix to run MR.
# The input files from I-TASSER-MR: 
#   REF_model[1-5].pdb:            the final built models of I-TASSER;
#   rep*tra*.bz2:                  decoys generate by I-TASSER;
#   combo[1-5].pdbcombo[1-5].pdb   cluster centroid of SPICKER
# The output files:
#   model[1-5].avs:                The AVS score of I-TASSER models;
#   *cutmodel.pdb:                 Search models produced according to the AVS score
#===================================================================================

use strict;
use warnings;
use Getopt::Long;
use Math::Trig;
use Statistics::Descriptive::Discrete;
use File::Spec;
######Usser should modify the following path#######################

my $path_curf = File::Spec->rel2abs(__FILE__);
my ($vol, $dirs, $file) = File::Spec->splitpath($path_curf);
$dirs =~ s/bin//g;
chop($dirs);

my $I_TASSER_MR_dir = $dirs;

##############################################################
my $output_dir  = "$I_TASSER_MR_dir/output";
my $MRdataset_dir = "$output_dir/MRDataset";
my $cluster_dir = "$output_dir/cluster";
my $bindir  = "$I_TASSER_MR_dir/bin";

my $operating_system=$^O;
unless ($operating_system eq "linux"){
    printf("Your operating system $operating_system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}

my ($targetname,$modelnum,$bfactor_type);
GetOptions('T:s' => \$targetname, 'N:s' => \$modelnum,'B:s' => \$bfactor_type);

if(!$targetname )
{
    printf "AVSprocess USAGE:\n";
    printf "Mandatory arguments:\n";
    printf "./AVSprocess.pl -T Target_name\n";
    printf "Optional argments:\n";
    printf "      -N modelnum: modelnum is a number from 1 to 5,which represents the model number of the I-TASSER Models.(default:0)\n";
    printf "      -B bfactor_type: 0: 20 is used(default) 1: ACCbfactor  2: AVSbfactor\n";
       
   exit();
}

if(!$modelnum){
   $modelnum=1;
}
if(!$bfactor_type){
   $bfactor_type=0;
}

my @ss      = ();
push( @ss, $targetname);

foreach my $s (@ss) {
    &bzPDB($s);
    &process( $s, $modelnum );
    my $tem = lc($s);
`$bindir/dsspcmbi "$cluster_dir/$s/REF_model$modelnum.pdb" "$cluster_dir/$s/model$modelnum.dssp" 2>> "$cluster_dir/$s/log.txt"`
      ;    # the same dir or not ?

     &produce_ACCBfactor( $s, $modelnum );

    if ( $bfactor_type == 0 ) {
            &process_Bfactor_20( $s, $modelnum );
    }
    elsif ( $bfactor_type == 1 ) {
        &processBfactor_acc( $s, $modelnum );

    }
    elsif ( $bfactor_type == 2 ) {

         &processBfactor_avs($s,$modelnum);
    }
    &process_search_model( $s, $modelnum );
}

sub process() {
    my ( $s, $modelnum ) = @_;

    my $combo;
    my @tem;
    my %distance;
    my $p  = 0;
    my $pn = 0;    #the length of protein
    my $ln = 0;    #the number of decoys
    open( $combo, "<$output_dir/cluster/$s/combo$modelnum.pdb" );
    while ( my $line = <$combo> ) {
        if ( substr( $line, 0, 4 ) eq 'ATOM' ) {
            $tem[$p] = substr( $line, 0, 26 );
            $p++;
        }
    }
    close($combo);

    my $str;
    my $num;
    my $rep;
    my $lineno = 0;
    open( $str, "<$output_dir/cluster/$s/str.txt" );
    while ( my $line = <$str> ) {

        if (   ( substr( $line, 2, 1 ) eq '1' )
            && ( substr( $line, 44, 3 ) eq 'rep' ) )
        {


            $num = substr( $line, 38, 4 );
            $rep = substr( $line, 44, 11 );
            my $repfile;
            my $seqlen;
            my @cordinate;
            open( $repfile, "<$output_dir/$s/$rep" );

            while ( my $line1 = <$repfile> ) {
                if ( substr( $line1, 31, 4 ) eq $num ) {
                    $seqlen = substr( $line1, 4, 4 );
                    for ( my $i = 0 ; $i < $seqlen ; $i++ ) {
                        $line1 = <$repfile>;
                        $cordinate[$i] = $line1;

                    }

                    open( WR, ">$output_dir/cluster/$s/tem$modelnum.pdb" );
                    for ( my $k = 0 ; $k < $p ; $k++ ) {
                        $cordinate[$k] =~ /\s+(\S+)\s+(\S+)\s+(\S+)\n/;
                        printf WR "%26s%12s%8s%8s\n", $tem[$k], $1, $2, $3;

                    }
                    close(WR);

`$bindir/myRMSD "$output_dir/cluster/$s/combo$modelnum.pdb" "$output_dir/cluster/$s/tem$modelnum.pdb" -o "$output_dir/cluster/$s/dis$modelnum.pdb"`;
                    my $disfile;
                    open( $disfile, "<$output_dir/cluster/$s/dis$modelnum.pdb" );
                    my @array;
                    $pn = 0;
                    while ( my $line2 = <$disfile> ) {
                        if ( substr( $line2, 0, 4 ) eq 'ATOM' ) {

                            $array[$pn] = substr( $line2, 55, 8 );

                            $pn++;
                        }
                    }
                    $distance{$ln} = \@array;
                    $ln++;
                }

            }
            close($repfile);

        }

    }
    close($str);
    my %distance_convert;

    #comput average value and output
    my $writeaver;
    open( $writeaver, ">$output_dir/cluster/$s/model$modelnum.avs" );
    for ( my $i = 0 ; $i < $pn ; $i++ ) {
        my @array2;
        for ( my $j = 0 ; $j < $ln ; $j++ ) {
            my $array = $distance{$j};
            $array2[$j] = $$array[$i];
        }
        $distance_convert{$i} = \@array2;
        my $array_forcompute = $distance_convert{$i};
        my $stats            = new Statistics::Descriptive::Discrete;
        $stats->add_data(@$array_forcompute);
        my $average = $stats->mean();
        my $dev     = $stats->standard_deviation();
        printf $writeaver "$average\t$dev\n";

    }
    close($writeaver);

}

sub bzPDB() {
    my ($s) = @_;
    my @bz_files = glob "$output_dir/$s/*.bz2";
    foreach my $bz (@bz_files) {
        `bunzip2 $bz`;
    }

}

sub produce_ACCBfactor() {
    my ( $s, $modelnum ) = @_;
    my $readdssp;
    open( WR,        ">$cluster_dir/$s/accbfactor$modelnum.txt" );
    open( $readdssp, "<$cluster_dir/$s/model$modelnum.dssp" );
    while ( my $line = <$readdssp> ) {
        if ( substr( $line, 2, 1 ) eq "#" ) {
            while ( $line = <$readdssp> ) {
                if ( length($line) > 10 ) {
                    my $str = substr( $line, 34, 4 );
                    $str =~ s/ //g;
                    printf WR "$str\n";

                }
            }
        }

    }

}

sub processBfactor_acc() {
    my ( $s, $modelnum ) = @_;

    my @bf;
    my $p = 0;
    my $read;
    open( $read, "<$cluster_dir/$s/accbfactor$modelnum.txt" );
    while ( my $line = <$read> ) {
        $bf[$p] = $line;
        $p++;
    }
    close($read);
    open( RD, "<$cluster_dir/$s/REF_model$modelnum.pdb" );

    #open( RD, "<$output_dir/$s/Zhang-Server_TS1" );
    print "$s\n";
    open( WR, ">$MRdataset_dir/$s/model$modelnum/BREF_model$modelnum.pdb" );

    # open( WR, ">$MRdataset_dir/$s/model$modelnum/BREF_model$modelnum.pdb" );
    my $q = 0;
    while ( my $line = <RD> ) {
        my $count = substr( $line, 22, 4 );
        $count =~ s/ //g;
        $q = $count - 1;
        my $bfactor;
        chomp( $bf[$q] );
        $bfactor = $bf[$q] + 10;
        $bfactor .= ".00";
        my $str = substr( $line, 0, 54 );

        if ( substr( $str, 0, 4 ) eq 'ATOM' ) {
            printf WR "$str  1.00 $bfactor\n";
        }
    }
    close(RD);
    close(WR);
}
sub processBfactor_avs() {
    my ( $s, $modelnum ) = @_;

    my @bf;
    my $p = 0;
    my $read;
    open( $read, "<$cluster_dir/$s/model$modelnum.avs" );
    while ( my $line = <$read> ) {
       $line=~ /(\S+)\t(\S+)\n/;
 
        $bf[$p] = $1;
        $p++;
    }
    close($read);
    open( RD, "<$cluster_dir/$s/REF_model$modelnum.pdb" );

    #open( RD, "<$output_dir/$s/Zhang-Server_TS1" );
    print "$s\n";
    open( WR, ">$MRdataset_dir/$s/model$modelnum/BREF_model$modelnum.pdb" );

    # open( WR, ">$MRdataset_dir/$s/model$modelnum/BREF_model$modelnum.pdb" );
    my $q = 0;
    while ( my $line = <RD> ) {
        my $count = substr( $line, 22, 4 );
        $count =~ s/ //g;
        $q = $count - 1;
        my $bfactor;
        chomp( $bf[$q] );
        $bfactor =8*3.14*3.14*$bf[$q]*$bf[$q]/3; 
         $bfactor=substr($bfactor,0,index($bfactor,".")+3);
        my $str = substr( $line, 0, 54 );

        if ( substr( $str, 0, 4 ) eq 'ATOM' ) {
            printf WR "$str  1.00 $bfactor\n";
        }
    }
    close(RD);
    close(WR);
}

sub process_Bfactor_20() {
    my ( $s, $modelnum ) = @_;
    open( RD, "<$cluster_dir/$s/REF_model$modelnum.pdb" );
    open( WR, ">$cluster_dir/$s/BREF_model$modelnum.pdb" );
    while ( my $line = <RD> ) {
        my $str = substr( $line, 0, 54 );
        if ( substr( $str, 0, 4 ) eq 'ATOM' ) {
            printf WR "$str  1.00 20.00\n";
        }

    }
    close(RD);
    close(WR);

}

sub process_search_model() {
    my ( $s, $modelnum ) = @_;

    my $cou = 0;

    #$s = substr( $s, 0, 5 );
    my @avger;
    my $p = 0;
    open( RD, "<$cluster_dir/$s/model$modelnum.avs" );
    while ( my $line = <RD> ) {
        $line =~ /(\S+)\t(\S+)\n/;
        $avger[$p] = $1;
        $p++;
    }
    close(RD);
    `mkdir -p $MRdataset_dir/$s/model$modelnum`;
    `cp $cluster_dir/$s/BREF_model$modelnum.pdb $MRdataset_dir/$s/model$modelnum/`;
    my $cutoff;
    if ( $p >= 100 ) {
        $cutoff = int( $p / 100 );
    }
    if ( $p < 100 ) {
        $cutoff = 1;
    }
    my $object = int( $p * 0.70 );    #object

    my $len = 0;

    my @bj;
    for ( my $i = 0 ; $i < $p ; $i++ ) {

        $len++;
        $bj[$i] = 1;

    }

    #printf "$len";
    my $molbj = 2;
    my $last  = 40;
    while ( $len > $object ) {
        $len = 0;
        my @maxvalue;
        $maxvalue[0] = $last;
        for ( my $j = 0 ; $j < $cutoff ; $j++ ) {
            my @temp;
            my $count = 0;
            for ( my $i = 0 ; $i < $p ; $i++ ) {
                if ( $avger[$i] < $maxvalue[$j] ) {

                    $temp[$count] = $avger[$i];
                    $count++;
                }

            }
            my $stats = new Statistics::Descriptive::Discrete;
            $stats->add_data(@temp);
            $maxvalue[ $j + 1 ] = $stats->max();
        }
        $last = $maxvalue[$cutoff];
        for ( my $j = 0 ; $j < $p ; $j++ ) {
            for ( my $i = 0 ; $i < $cutoff + 1 ; $i++ ) {
                if ( $avger[$j] eq $maxvalue[$i] ) {
                    $bj[$j] = 0;
                }
            }
        }
        my $name;
        $name = "$molbj";
        $name .= "cutmodel.pdb";
        open( WR, ">$MRdataset_dir/$s/model$modelnum/$name" );
        for ( my $i = 0 ; $i < $p ; $i++ ) {
            if ( $bj[$i] eq '1' ) {
                $len++;
                open( RD, "<$cluster_dir/$s/BREF_model$modelnum.pdb" );
                while ( my $line1 = <RD> ) {
                    if ( substr( $line1, 0, 4 ) eq "ATOM" ) {
                        my $str1 = substr( $line1, 22, 4 );
                        $str1 =~ s/ //g;
                        if ( $str1 eq ( $i + 1 ) ) {

                            printf WR "$line1";
                        }
                    }

                }
                close(RD);
            }
        }
        close(WR);
        $molbj++;

    }

}

sub readconfig(){


}

