#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;
use File::Spec;

####Modify the following path#################################
my $path_curf = File::Spec->rel2abs(__FILE__);
my ($vol, $dirs, $file) = File::Spec->splitpath($path_curf);
$dirs =~ s/bin//g;
chop($dirs);
my $I_TASSER_MR_dir = $dirs;

my $user=`whoami`;
chop($user);
my $tmp_dir = "/tmp/$user/I-TASSER-MR";

##############################################################

my $operating_system=$^O;
unless ($operating_system eq "linux"){
    printf("Your operating system $operating_system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}

my ($target,$modelnum,$isCluster);
GetOptions('T:s' => \$target, 'N:s' => \$modelnum,'C:s' => \$isCluster);

if(!$target )
{
    printf "runBuild USAGE:\n";
    printf "Mandatory arguments:\n";
    printf "./runBuild.pl -T Target_name\n";
    printf "Optional argments:\n";
    printf "      -N modelnum: modelnum is a number from 1 to 5,which represents the model number of the I-TASSER Models.(default:1)\n";
    printf "      -C isCluster: 0(default): jobs running locally; 1:some job will submitted to PBS cluster\n";
       
   exit();
}
if(!$modelnum){
   $modelnum=1;
}
if(!$isCluster){
   $isCluster=0;
}


my $output_dir = "$I_TASSER_MR_dir/output/MRESULT/$target/model$modelnum";

my $MRESULT_dir = "$I_TASSER_MR_dir/output/MRESULT/$target/model$modelnum";
open( RD, "<$output_dir/mgoodfor_build.txt" );
my @ss;
my @mol;
my $count = 0;
open( RD, "<$output_dir/mgoodfor_build_f.txt" );
while ( my $line = <RD> ) {
    $ss[$count] = substr( $line, 0, index( $line, "/" ) );
    $line =~ /\S+\s+(\S+).*/;
    $mol[$count] =
      substr( $1, index( $1, "/" ) + 1, length($1) - index( $1, "/" ) - 1 );
    $count++;
}
close(RD);

open( WRMOL, ">$output_dir/model_name.txt" );
for ( my $i = 0 ; $i < $count ; $i++ ) {
    my $tem = "AutoMR_";

    # print "$ss[$i]\n";
    $ss[$i] =~ /(\S+)\s(\S+)/;
    my $protein = $1;
    my $tmp2    = $2;
    my $name    = substr( $protein, 2, 5 );

    #print "$name\n";

    my $run = substr( $tmp2, 7, length($tmp2) - 7 );
    printf WRMOL "$protein-$run $mol[$i]\n";
    $tem .= $run;

    #print "$output_dir2/$protein/$tem/MR.1_refine_001.pdb\n";
    if(-e "$output_dir/$protein/$tem/MR.1_refine_001.pdb")
{
    open( RD, "<$output_dir/$protein/$tem/MR.1_refine_001.pdb" );
    open( WR, ">$output_dir/$protein/$tem/MR.1_refine_002.pdb" );
    while ( my $line = <RD> ) {
        if ( substr( $line, 0, 4 ) eq 'ATOM' ) {
            printf WR "$line";
        }
    }
    close(WR);
    close(RD);
}
    open( WR, ">$output_dir/$protein-$run.sh" );
   if(-e "$output_dir/$protein/$tem/MR.1_refine_001.pdb")
  { 
   printf WR
"phenix.autobuild data=$name.mtz model=MR.1_refine_002.pdb seq_file=seq.fasta rebuild_in_place=False";
  }else 
   {
      printf WR
     "phenix.autobuild data=$name.mtz model=MR.1.pdb seq_file=seq.fasta rebuild_in_place=False";
   }
    close(WR);

}
close(WRMOL);


my @ss2 = glob "$MRESULT_dir/*.sh";

foreach my $s (@ss2) {
    my @tt = split( '/', $s );
    my $jobname = substr( $tt[-1], 0, length( $tt[-1] ) - 3 );
    my $job = `cat $I_TASSER_MR_dir/bin/BUILDmod`;
    my $protein = $target;
    $jobname =~ /(\S+)-(\S+)*/;
    my $tem1 = $1;
    $tem1 .= "/AutoMR_";
    $tt[-1] =~ /(\S+)\-(\S+).sh/;
    $tem1 .= $2;
    $job =~ s/#JOBNAME#/$jobname/mg;
    $job =~ s/#I-TASSER-MR#/$I_TASSER_MR_dir/mg;
    $job =~ s/#WORKDIR#/$tmp_dir/mg;
    $job =~ s/#MODEL#/$protein/mg;
    $job =~ s/#EXPERIMENT#/$tem1/mg;
    $job =~ s/#MODELNUM#/$modelnum/mg;
    open( WR, ">$MRESULT_dir/$jobname.pl" );
    printf WR $job;
    close(WR);
    `chmod +x $MRESULT_dir/$jobname.pl`;
    
    if ( $isCluster == 1 ) {
`qsub -q urgent -l walltime=300:00:00 -e $MRESULT_dir/err_$jobname -o $MRESULT_dir/out_$jobname $MRESULT_dir/$jobname.pl`;
    }
    elsif ( $isCluster == 0 ) {
        `$MRESULT_dir/$jobname.pl`;
    }
}

