#!/usr/bin/perl 
#===============================================================================
#===============================================================================

use strict;
use warnings;
use Getopt::Long;
use Statistics::Descriptive::Discrete;
use File::Spec;
########Change the following path,please!###########################

my $path_curf = File::Spec->rel2abs(__FILE__);
my ($vol, $dirs, $file) = File::Spec->splitpath($path_curf);
$dirs =~ s/bin//g;
chop($dirs);

my $I_TASSER_MR_dir = $dirs;

my $user=`whoami`;
chop($user);
my $tmp_dir = "/tmp/$user/I-TASSER-MR";

#####################################################################
my $operating_system=$^O;
unless ($operating_system eq "linux"){
    printf("Your operating system $operating_system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}

my ($targetname,$modelnum,$isCluster);
GetOptions('T:s' => \$targetname, 'N:s' => \$modelnum,'C:s' => \$isCluster);

if(!$targetname )
{
    printf "runMR USAGE:\n";
    printf "Mandatory arguments:\n";
    printf "./runMR.pl -T Target_name\n";
    printf "Optional argments:\n";
    printf "      -N modelnum: modelnum is a number from 1 to 5,which represents the model number of the I-TASSER Models.(default:1)\n";
    printf "      -C isCluster: 0(default): jobs running locally; 1:some job will submitted to PBS cluster\n";
       
   exit();
}

if(!$modelnum){
   $modelnum=1;
}
if(!$isCluster){
   $isCluster=0;
}


#####################################################################
my $I_TASSER_MR_BIN_dir   = "$I_TASSER_MR_dir/bin";
my $output_dir   = "$I_TASSER_MR_dir/output";
my $MRdataset_dir  = "$output_dir/MRDataset";
my $s        = $targetname; #target name
my $tem      = lc($s);
&produce_mrjoblist( $s, $modelnum );
&qsubjobs( $s, $modelnum );


#####################################################################u

sub produce_mrjoblist() {
    my ($s)    = @_;
    my $number = `ls -1 $MRdataset_dir/$s/model$modelnum/*cutmodel*|wc -l`;
    my @model  = glob "$MRdataset_dir/$s/model$modelnum/*cutmodel.pdb";

    my $space = '';

    my $unit_cell;
    my $length;
    my $rms;
    my $copy   = '';
    my $params = `cat $output_dir/$s/params.txt`;
    chomp($params);

    #print("$params\n");
    $params =~
      /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/;
    $unit_cell  = " $1 $2 $3 $4 $5 $6";
    $rms = $7;
    $space = $8;
    $copy = $9;


    # chomp($space);
    if ( $copy == 0 ) {
        $copy = &getCopyValue( $unit_cell, $space, $length );
    }

    my $seq = `cat $output_dir/$s/$s.fasta`;
    $seq =~ s/\s//mg;
    $length = length($seq);
    $length *= 100;

    my $cutoff = int( $number / 5 );
    my $left   = $number % 5;
    open( WRlist, ">$MRdataset_dir/$s/model$modelnum/mr_list.txt" );
    for ( my $j = 0 ; $j < 5 ; $j++ ) {

        my $mol = "$j";
        $mol .= "_";
        $mol .= $s;

        open( WR, ">$MRdataset_dir/$s/model$modelnum/$mol-run.sh" );
        printf "$MRdataset_dir/$s/model$modelnum/$mol-run.sh\n";
        printf WR "#!/bin/sh\n";
        for ( my $k = $cutoff * $j ; $k < $cutoff * ( $j + 1 ) ; $k++ ) {
            my $runnum;
            $runnum = $k % $cutoff;
            $runnum = $runnum + 1;
       #     print "$model[$k]\n";
            my @tem_mol = split( '/', $model[$k] );
            my $tem3 = $tem_mol[-1];
      #      print "$tem3\n";
            printf WR
"phenix.automr $tem3 $s.mtz mass=$length.  \\\n";
            printf WR
              "component_type=protein RMS=$rms seq.fasta copies=$copy  \\\n";
            printf WR "space_group=$space \\\n";
            printf WR "unit_cell=\"$unit_cell\" \\\n";
            printf WR "rebuild_after_mr=False\n";
            my $tem = "AutoMR_run_";
            $tem .= "$runnum";
            $tem .= "_";
            printf WR
"phenix.refine $s.mtz $tem/MR.1.pdb main.number_of_macro_cycles=20 refinement.input.xray_data.labels=\"FP,SIGFP\"\n";
            printf WR "mv *refine* $tmp_dir/model$modelnum/$mol/$tem/ \n\n";
            printf WRlist "$mol\t$tem\n";
        }
        close(WR);

    }

    # print the left modelsA

    my $lmol = "5";
    $lmol .= "_";
    $lmol .= $s;
    open( WRLEFT, ">$MRdataset_dir/$s/model$modelnum/$lmol-run.sh" );
    printf WRLEFT "#!/bin/sh\n";
    my $lrunnum;

    for ( my $k = $cutoff * 5 ; $k < $number ; $k++ ) {
        $lrunnum = $k % $cutoff;
        $lrunnum = $lrunnum + 1;
        my @tem_mol = split( '/', $model[$k] );
        my $tem3 = $tem_mol[-1];

        #    print "$tem3\n";
        printf WRLEFT
"phenix.automr $tem3 $s.mtz mass=$length.  \\\n";
        printf WRLEFT
          "component_type=protein RMS=$rms seq.fasta copies=$copy  \\\n";
        printf WRLEFT "space_group=$space \\\n";
        printf WRLEFT "unit_cell=\"$unit_cell\" \\\n";
        printf WRLEFT "rebuild_after_mr=False\n";
        my $tem = "AutoMR_run_";
        $tem .= "$lrunnum";
        $tem .= "_";
        printf WRLEFT
"phenix.refine $s.mtz $tem/MR.1.pdb main.number_of_macro_cycles=20 refinement.input.xray_data.labels=\"FP,SIGFP\"\n";
        printf WRLEFT
"mv *refine* $tmp_dir/model$modelnum/$lmol/$tem/ \n\n";
        printf WRlist "$lmol\t$tem\n";

    }
    $lrunnum = $lrunnum + 1;
    printf WRLEFT
"phenix.automr BREF_model$modelnum.pdb $s.mtz mass=$length.  \\\n";
    printf WRLEFT
      "component_type=protein RMS=$rms seq.fasta copies=$copy  \\\n";
    printf WRLEFT "space_group=$space \\\n";
    printf WRLEFT "unit_cell=\"$unit_cell\" \\\n";
    printf WRLEFT "rebuild_after_mr=False\n";
    my $ltem = "AutoMR_run_";
    $ltem .= "$lrunnum";
    $ltem .= "_";
    printf WRLEFT
"phenix.refine $s.mtz $ltem/MR.1.pdb main.number_of_macro_cycles=5 refinement.input.xray_data.labels=\"FP,SIGFP\"\n";
    printf WRLEFT
      "mv *refine* $tmp_dir/model$modelnum/$lmol/$ltem/ \n\n";
    printf WRlist "$lmol\t$ltem\n";
    close(WRLEFT);

}
close(WRlist);

sub qsubjobs() {
    my ( $s, $modelnum,$iscluster) = @_;

    my $number = `ls -1 $MRdataset_dir/$s/model$modelnum/*cutmodel*|wc -l`;
    `mkdir -p $output_dir/MRESULT/$s/model$modelnum/record`;

    for ( my $j = 0 ; $j < 6 ; $j++ ) {

        my $job = `cat $I_TASSER_MR_BIN_dir/MRmod`;

        my $tem2 = "$j";
        $tem2 .= "_";
        $tem2 .= $s;
        my $jobname = "$tem2";

        $job =~ s/#WORKDIR#/$tmp_dir/mg;
        $job =~ s/#I_TASSER_MR_DIR#/$I_TASSER_MR_dir/mg;
        $job =~ s/#JOBNAME#/$jobname/mg;
        $job =~ s/#MODEL#/$tem2/mg;
        $job =~ s/#EXPERIMENT#/$s/mg;
        $job =~ s/#TARGETNAME#/$s/mg;
        $job =~ s/#MODELNUM#/$modelnum/mg;
        open( WR, ">$output_dir/MRESULT/$s/model$modelnum/record/$jobname.pl" );
        printf WR "$job";
        close(WR);
        `chmod +x $output_dir/MRESULT/$s/model$modelnum/record/$jobname.pl`;
    if($isCluster==1){
    `qsub -q urgent -l walltime=300:00:00 -e $output_dir/MRESULT/$s/model$modelnum/record/err_$jobname -o $output_dir/MRESULT/$s/model$modelnum/record/out_$jobname  $output_dir/MRESULT/$s/model$modelnum/record/$jobname.pl`;

     print("$jobname is submitted!\n");
    
   }elsif($isCluster==0){
     print("Running $output_dir/MRESULT/$s/model$modelnum/record/$jobname.pl\n");
    `$output_dir/MRESULT/$s/model$modelnum/record/$jobname.pl`;
   }
}
}
